function [tx,ty,tz,epx,epy,epz]=GetTetErrorEllipsoids(wf,r1,r2,r3,r4,v1,v2,v3,v4)
% [tx,ty,tz,epx,epy,epz]=GetTetErrorEllipsoids(wf,r1,r2,r3,r4,v1,v2,v3,v4);
% Make a 4D ellipsoid (from weber fractions) and % then convert to 
% tetrahedral coordinates
% uses functions Get4dEllipsoid.m and totet.m (must be in same directory)
% January 2009
% Please send corrections to John A. Endler   J.A.Endler@exeter.ac.uk
%
% INPUT
%   wf           Weber Fraction (usually about 0.05 in vertebrates)
%   r1,r2,r3,r4  Relative abundances of UVS, SWS, LWS, MWS cone types
%                  for example 1,2,2,4
%   v1,v2,v3,v4  Photon captures of the 4 cone types
% OUTPUT
%   tx,ty,tz     Stimulus mean point = ellipsoid center
%   epx,epy,epz  Points defining the error ellipsoid in tetrahedral space
%                 plot with plot3(epx,epy,epz)
%
w1=wf/sqrt(r1); w2=wf/sqrt(r2); w3=wf/sqrt(r3); w4=wf/sqrt(r4);
%ensure that sum of cone outputs equals 1
t=v1+v2+v3+v4; va=v1/t; vb=v2/t; vc=v3/t; vd=v4/t; clear t;
% Put points on the edge of a 4D ellipsoid into coordinates (a,b,c,d).
[a,b,c,d]=Get4dEllipsoid(w1,w2,w3,w4); [n,i]=size(a);
fprintf(1,'Extracting tetrahedral ellipsoid (takes a while)\n');
% Set nonzero center of hypersphere (not black color)
%   and set negative values to zero (i.e. not sensed)
%   and remove points with row sum=0
k=0;
for i=1:n
  ta=a(i)+va; tb=b(i)+vb; tc=c(i)+vc; td=d(i)+vd;  
  if ta<0 ta=0; end; if tb<0 tb=0; end;
  if tc<0 tc=0; end; if td<0 td=0; end;
  tt=ta+tb+tc+td;
  if tt>0 k=k+1; u(k)=ta; s(k)=tb; m(k)=tc; l(k)=td; end;
end;
u=u'; s=s'; m=m'; l=l'; n=k; 
clear ta tb tc td tt k;
% Now convert each 4D point in ellipsoid to tetrahedral coordinates
[tx,ty,tz]=totet(va,vb,vc,vd); %the point representing the average stimulus
[px,py,pz]=totet(u,s,m,l);     %the ellipsoid
clear u s m l a b c d;

%now find convex hull (reduce number of points) of tx,ty,tz
elp=[px py pz];
K=convhulln(elp); k=K(:); [n,i]=size(k); %collapse to point list
%now extract duplicate points
clear K; done=zeros(n,i);
epn=0;
for i=1:n
  if done(i)==0 
        epn=epn+1; j=k(i);
        epx(epn)=elp(j,1); epy(epn)=elp(j,2); epz(epn)=elp(j,3);
        done(i)=1;
  end;
end;
fprintf(1,'Finished\n');
end

function [a,b,c,d]=Get4dEllipsoid(r1,r2,r3,r4)
% [a,b,c,d]=Get4dEllipsoid(r1,r2,r3,r4);
% Create points in and on the surface of a 4-dimenstional  
%   ellipsoid of radii r1 r2 r3 r4 centered on the origin.
% Result is 4 column vectors, a,b,c,d -- the coordinates.
% As a check, plotting any three of these four coordinates with 
%   Plot3(x,y,z,'.') gives a 3D ellipsoid outlined in points.
% NOTE: this can be slow
% NOTE: owing to rounding errors, actual limits may be
%   less than -r to r  (need to put in missing points)
s=15; %s = number of sectors
cx=(-1:1/s:1)'; [s,i]=size(cx); i=s*s*s;
fprintf(1,'Calculating 4D ellipsoid (this takes a while)\n');
n=0; 
for i=1:s
    %fprintf(1,'i = %u of %u \n',i,s);
  for j=1:s 
    for k=1:s
        t=1-cx(i)^2-cx(j)^2-cx(k)^2;
        if t>0.0001
          n=n+1; x(n)=cx(i); y(n)=cx(j); z(n)=cx(k); w(n)=sqrt(t); 
          n=n+1; x(n)=cx(i); y(n)=cx(j); z(n)=cx(k); w(n)=-w(n-1); 
        end;
    end;
  end;
end;
a=x'; b=y'; c=z'; d=w';
%this has a radius of 1, now convert to radius r1, r2, r3, r4
a=r1*a; b=r2*b; c=r3*c; d=r4*d;
%fprintf(1,'%u points in 4D ellipsoid, coordinates (a,b,c,d) \n',n);
end

function [TX,TY,TZ]=totet(UVS,SWS,MWS,LWS)
% [tx,ty,tz]=ToTet(UV,SW,MW,LW);  converts cone data to 
%   tetrahedron coordinates.  First makes UVS+SWS+MWS+LWS=1
% Plot in the tetrahedron using plot3 or scatter3(TX,TY,TZ...)
%   Use tetra=drawtet; to draw tetrahedron first.
tot=UVS+SWS+MWS+LWS;
if tot<=0 error('Zero total cone output'); end;
UVS=UVS./tot; SWS=SWS./tot; MWS=MWS./tot; LWS=LWS./tot;
[n,j]=size(UVS); clear j;
TX=zeros(n,1); TY=TX; TZ=TX;
for i=1:n
  TY(i)=(1/2)*sqrt(3/2)*(UVS(i)+2*SWS(i)+MWS(i)-1); 
  TX(i)=(UVS(i)+3*MWS(i)-1)/(2*sqrt(2));
  TZ(i)=UVS(i)-(1/4);
end;
end

